/*
 * windex.c
 *
 * Simple Tahoma Tech 10120/10121 test program.  Connects to 120/121
 * via TCP/IP using internal Axis 1800 or external Axis 5400+ print
 * server and sends test patterns and files via command line.
 * 
 * Tahoma Technology (formerly Ikon Corporation)
 * 2819 Elliott Avenue, Suite 203
 * Seattle, WA, USA, 981219
 *
 * 206.728.6465
 * http://www.tahomatech.com
 * tahoma@tahomatech.com
 *
 * see windex.h for version and change list
 */

#include "./windex.h" 

/*
 * simple replacement for getopt fcn for use in win apps where
 * unix getopt not available
 *
 * return single char option if match, '?' if no match, EOF if
 * arg list exhausted, and *optarg set to option arg if used
 * if option requires argument, but list exhausted, '?' returned
 * unless 1st char of opt string is ':', in which case ':'
 * returned
 *
 * this simplified version only allows one option per "-"
 * multiple options require "-x -y ... -x" form
 * options w/args may use either "-x arg" or "-xarg" form
 *
 * on exit, optind points to next argv element
 *
 * global optind MUST be initialized to 1!
 */

static int getcmdopt(int argc, char * const argv[], const char *optstring)
{
	char *curarg;
	int missingarg;

	if(optind >= argc)
		return EOF;

	curarg = argv[optind];
	if(*curarg != '-') {
		optarg = NULL;
		return EOF;
	}
	curarg++;
	optopt = *curarg;
	missingarg = (*optstring == ':') ? ':' : '?';
	while(*optstring) {
		if(*optstring == *curarg) {
			/* option match */
			if(*(optstring + 1) == ':') {
				/*
				 * arg required
				 * if next char in curarg not null
				 * return ptr to _that_ as arg
				 * otherwise use next argv
				 */
				if(*(curarg + 1) != '\0') {
					optarg = curarg + 1;
					optind++;
					return *curarg;
				}
				if(++optind >= argc) {
					optarg = NULL;
					return missingarg;
				}
				optarg = argv[optind];
				optind++;
				return *curarg;
			}
			optarg = NULL;
			optind++;
			return *curarg;
		}
		optstring++;
	}
	optopt = *curarg;
	return '?';
}



/*
 * get next option (and possibly an argument) from interactive
 * input
 *
 * make it look like cmd line getcmdopt to extent possible
 *
 * only one option per line allowed
 */

static int getinteropt(char *optstring)
{
	char linestr[132];
	char cstr[8];
	char *lineptr;
	int c;
	int missingarg;

	/*
	 * scan for the option flag and argument
	 * avoid any leading '-' and if s (string)
	 * scan for characters between double quotes
	 */
	printf("enter command (h = help, q = quit): ");
	strcpy(linestr,"?");
	strcpy(cstr,"?");
	strcpy(loptstr,"");
	fgets(linestr, 130, stdin);
	if((lineptr = strchr(linestr, '-')) != NULL)
		lineptr++;
	else
		lineptr = linestr;
	sscanf(lineptr, "%1s %130s", cstr, loptstr);
	c = cstr[0];
	optopt = c;
	missingarg = (*optstring == ':') ? ':' : '?';
	while(*optstring != c) {
		if(*optstring == '\0') {
			/* no match */
			loptarg = NULL;
			return '?';
		}
		optstring++;
	}
	/* optstring points at matching option
	 * test for '"' in input string
	 * if so, arg is all within quotes
	 * if not, arg already in loptstr
	 */
	if(*(optstring + 1) == ':') {
		if(strchr(lineptr, '"'))
			sscanf(lineptr, "%*1s \"%[^\"]", loptstr);
		loptarg = loptstr;
		/* test for missing arg */
		if(!strlen(loptarg)) {
			loptarg = NULL;
			return missingarg;
		}
	}
	return c;
}



/*
 * get next option and arg from command line or input stream
 *
 * return from getcmdopt or getinteropt passed back to caller:
 * if unknown option, return '?', if missing arg, return ':'
 * (if opstring begins w/':'), if no further options, return EOF
 *
 * if option arg used, loptarg point to arg string
 */

static int getcommand(int argc, char * const argv[], char *optstring)
{
	int c;

	if(interactive) {
		c = getinteropt(optstring);
		/* getinteropt sets loptarg */
	}
	else {

		c = getcmdopt(argc, argv, optstring);
		loptarg = optarg;
	}
	/* solaris printf segfaults on null %s */
	if(verbose)
		printf("(getcommand) option = \"%c\", arg = \"%s\"\n",
			(c != EOF) ? c : ' ', loptarg ? loptarg : "(NULL)");
	return c;
}



/*
 * clean up before error exit or quit
 */

static void cleanup(void) {

	if(locbuffptr != NULL)
		free(locbuffptr);
	if(filedes != -1)
		close(filedes);
	if(connected)
		shutdown(sockfd, 2);
	if(sockfd != INVALID_SOCKET)
		closesocket(sockfd);
#ifdef _WINBUILD_
	if (WSAstarted)
		WSACleanup();
#endif
}



/*
 * in interactive mode we return to input mode and
 * wait for quit or other command
 *
 * in command line mode, just clean up and exit
 */

static void errexit(void)
{
	if(!interactive) {
		cleanup();
		exit(EXIT_FAILURE);
	}
}


/*
 * check for connection
 * do the usual if not connected
 */

static int checkconnect(void)
{
	if(connected)
		return 0;
	printf("(checkconnect) no connection to device\n");
	errexit();
	return -1;
}



/*
 * open file
 * typical error stuff if failure
 */

static int openfile(void)
{
	filedes = open(loptarg, O_RDONLY | O_BINARY);
	if(filedes == -1) {
		printf("(openfile) can't open file \"%s\"\n", loptarg);
		errexit();
		return -1;
	}
	return 0;
}


/*
 * print the rules
 */

static void usage(void)
{
	char dummystr[8];
	
	printf("\nUsage: windex [-d<>] [-a<>] [-m<>] [-f<>] [-r<>] [-b<>] [-t<>] [-c<>]\n");
	printf("              [-s<>] [-n] [-p] [-v] [-i] [-h] [-q]\n");
	printf("        -d<output device port> set device port - e.g. -d 9100\n");
	printf("          optional - set before -a<> if used - defaults to 9900\n");
	printf("          typically use 9900 for Axis server, 9100 for SEH server\n");
	printf("        -a<output device address> (required - e.g. 192.168.1.123)\n");
	printf("        -m<mode> mode for transfer, PRINT, PLOT, SPP_PRINT, SPP_PLOT\n");
	printf("        -f<filename> send data file using esc wrapper\n");
	printf("        -r<filename> send raw data file w/out esc wrapper\n");
	printf("        -b<byte> send single byte - e.g. -b 0x55\n");
	printf("          or -b \"byte count\" repeats byte count times\n");
	printf("          or -b \"byte ... byte count\" repeats multi-byte pattern count times\n");
	printf("                **** double quotes required with count option ****\n");
	printf("        -t<count> send count pairs of repeating 0x55 0xAA test pattern bytes\n");
	printf("\n--More-- (enter)\n");
	fgets(dummystr, 2, stdin);
	printf("        -c<command> remote command to board/plotter, can be:\n");
	printf("                 SPEED_0 - slowest handshake speed\n");
	printf("                 SPEED_1\n");
	printf("                 SPEED_2\n");
	printf("                 SPEED_3 - fastest handshake speed\n");
	printf("                 VPRINT - set Versatec print mode\n");
	printf("                 VPLOT - set Versatec plot mode\n");
	printf("                 VSPP_PRINT - set Versatec SPP print mode\n");
	printf("                 VSPP_PLOT - set Versatec SPP plot mode\n");
	printf("                 VRST - Versatec Remote Reset\n");
	printf("                 VCLR - Versatec Remote Clear\n");
	printf("                 VFED - Versatec Remote Form Feed\n");
	printf("                 VEOT - Versatec End or Terminate\n");
	printf("                 VLTR - Versatec Remote Line Terminate\n");
	printf("\n--More-- (enter)\n");
	fgets(dummystr, 2, stdin);
	printf("        -s<string> output string to plotter (in current print/plot mode)\n");
	printf("                 **** strings must be in double quotes ****\n");
	printf("        -n output carriage return/line feed pair of ascii characters\n");
	printf("                 **** requires print mode ****\n");
	printf("        -p output carriage return/form feed pair of ascii characters\n");
	printf("                 **** requires print mode ****)\n");
	printf(" \n");
	printf("        -v toggle verbose mode\n");
	printf("        -i enter interactive mode.  in interactive mode all commands may\n");
	printf("                be entered **one per line** w/or w/out the leading -\n");
	printf("        -h print this help info\n");
	printf("        -q exit interactive mode\n");
	printf("\n--More-- (enter)\n");
	fgets(dummystr, 2, stdin);
	printf(" \n");
	printf("Note: commands are executed left to right as found, multiple files in \n");
	printf("      different modes can be sent in a single command\n");
	printf("      e.g. windex -a192.168.1.123 -mPRINT -ffile1.prn -mPLOT -ffile2.plt\n");
	printf("      sends file1.prn in print mode, and then file2.plt in plot mode\n");
	printf(" \n");
	printf("      in interactive mode, enter one command per line\n");
	printf(" \n");
}



/*
 * open a socket and connect to requested device
 * return 0 if success, -1 if not (unless in cmd line
 * mode, in which case errexit will abort)
 */

static int socketconnect(void) {
	if(sockfd != INVALID_SOCKET) {
		printf("(socketconnect) socket already open\n");
		printf("(socketconnect) chose other option, or exit and restart\n");
		errexit();
		return -1;
	}
	if(verbose)
		printf("(socketconnect) connecting to device \"%s\"\n", loptarg);
	if(connected) {
		printf("(socketconnect) device connection already established\n");
		printf("(socketconnect) chose other option, or exit and restart\n");
		errexit();
		return -1;
	}
	dev_inet_addr = inet_addr(loptarg);
	if(dev_inet_addr == INADDR_NONE) {
		printf("(socketconnect) \"%s\" is not a valid device address\n", loptarg);
		errexit();
		return -1;
	}
	memset((char *)&device_addr, 0, sizeof(device_addr));
	device_addr.sin_family = AF_INET;
	device_addr.sin_addr.s_addr = dev_inet_addr;
	device_addr.sin_port = htons(dev_port);
	sockfd = socket(AF_INET, SOCK_STREAM, 0);
	if(sockfd == INVALID_SOCKET) {
		printf("(socketconnect) can't open streaming socket\n");
		errexit();
		return -1;
	}
	if(connect(sockfd, (struct sockaddr *)&device_addr, sizeof(device_addr))) {
		printf("(socketconnect) can't connect to device \"%s\"\n", loptarg);
		closesocket(sockfd);
		sockfd = INVALID_SOCKET;
		errexit();
		return -1;
	}
	connected = 1;
	if(verbose)
		printf("(socketconnect) connected to \"%s\"\n", loptarg);
	return 0;
}


#ifdef _WINBUILD_
/*
 * start windows socket services @ requested version level
 * just bail out the side w/error if failure
 */

static void winsockstart(void)
{
	/* creepy windows looking variables */
	WORD wVersionRequested;
	BYTE wsamajor = WSAMAJOR;
	BYTE wsaminor = WSAMINOR;

	wVersionRequested = MAKEWORD( wsamajor, wsaminor );
	if(WSAStartup( wVersionRequested, &wsaData )) {
		printf("(winsockstart) no suitable winsock support available\n");
		exit(EXIT_FAILURE);
	}
	if((LOBYTE(wsaData.wVersion) != WSAMAJOR) ||
			(HIBYTE(wsaData.wVersion) != WSAMINOR)) {
		printf("(winsockstart) no suitable winsock version available\n");
		WSACleanup();
		exit(EXIT_FAILURE);
	}
}
#endif


/*
 * send count bytes from buffer
 * loop since send may return success but not xfer
 * all the requested bytes
 * do usual error return (exit if cmd line, return
 * -1 if interactive)
 */

static int senddata(char *buffer, int count)
{
	int sendresult;

	if((count > MAXBUFF) || (count <= 0)) {
		printf("(senddata) send byte count %d out of range\n", count);
		errexit();
		return -1;
	}
	while(count) {
		sendresult = send(sockfd, buffer, count, 0);
		if(sendresult == SOCKET_ERROR) {
			printf("(senddata) send error\n");
			errexit();
			return -1;
		}
		buffer += sendresult;
		count -= sendresult;
	}
	return 0;
}



/*
 * send file to socket using "pass for count"
 * called fcns will exit directly if error and
 * not in interactive mode
 *
 * code almost the same as sendrawfile except adds
 * escape and count at beginning of buffer
 */

static int sendwrappedfile(void)
{
	int count;
	long total = 0;
	time_t starttime;

	if(verbose) {
		printf("(sendwrappedfile) sending file \"%s\" in wrapper\n", loptarg);
		printf("(sendwrappedfile) one '*' per senddata call\n");
	}
	if(checkconnect())
		return -1;
	if(openfile())
		return -1;
	starttime = time(NULL);
	while(1) {
		count = read(filedes, locbuffptr + LOCOFFSET, LOCDATASIZE);
		if(count == 0)
			break;
		if(count == -1) {
			printf("(sendwrappedfile) read error after %d data bytes sent\n",
					total);
			close(filedes);
			errexit();
			return -1;
		}
		/* plug in escape and size and send including
		 * pass escape and count
		 */
		*locbuffptr       = BA_ESC;
		*(locbuffptr + 1) = PASS;
		*(locbuffptr + 2) = (count - 1) & 0xFF;
		*(locbuffptr + 3) = ((count -1) >> 8) & 0xFF;
		if(senddata(locbuffptr, count + LOCOFFSET)) {
			printf("(sendwrappedfile) senddata error after %d data bytes sent\n",
					total);
			close(filedes);
			errexit();
			return -1;
		}
		total += count;
		if(verbose) {
			printf("*");
			fflush(stdout);
		}
	}
	if(verbose)
		printf("\n(sendwrappedfile) done - %d data bytes sent in %d seconds\n",
				total, time(NULL) - starttime);
	close(filedes);
	return 0;
}



/*
 * send raw file to socket direcly w/out checking
 * for 0xBA in data - 0xBA in file will cause unpredictable
 * behavior
 *
 * we will use the same max and buff offset as sendwrappedfile
 * and maybe combine code (we _could_ use whole buffer here)
 *
 * called fcns will exit directly if error and
 * not in interactive mode
 */

static int sendrawfile(void)
{
	int count;
	long total = 0;
	time_t starttime;

	if(verbose) {
		printf("(sendrawfile) sending raw file \"%s\"\n", loptarg);
		printf("(sendrawfile) one '*' per senddata call\n");
	}
	if(checkconnect())
		return -1;
	if(openfile())
		return -1;
	starttime = time(NULL);
	while(1) {
		count = read(filedes, locbuffptr + LOCOFFSET, LOCDATASIZE);
		if(count == 0)
			break;
		if(count == -1) {
			printf("(sendrawfile) read error after %d bytes sent\n",
					total);
			close(filedes);
			errexit();
			return -1;
		}
		if(senddata(locbuffptr + LOCOFFSET, count)) {
			printf("(sendrawfile) senddata error after %d bytes sent\n",
					total);
			close(filedes);
			errexit();
			return -1;
		}
		total += count;
		if(verbose) {
			printf("*");
			fflush(stdout);
		}
	}
	if(verbose)
		printf("\n(sendrawfile) done - %d bytes sent in %d seconds\n",
				total, time(NULL) - starttime);
	close(filedes);
	return 0;
}



/*
 * sanity wraper for strtol
 *
 * get long integer argument from arg string
 * auto radix detection
 * sets global longarg if valid string
 * 
 * return 1 if legit #, 0 if EOL or newline, -1
 * if invalid input string
 */

static int getlongarg(char *instring, char **endstring)
{
	unsigned long templong;
	char *endptr;

	if((*instring == '\0') || (*instring == '\n'))
		return 0;
	templong = strtol(instring, &endptr, 0);
	if(endstring != NULL)
		*endstring = endptr;
	if(verbose)
		printf("(getlongarg) argstring = \"%s\", result (decimal) = %d\n",
				instring, templong);
	/* if endptr changed, it did a legit conversion */
	if(endptr != instring) {
		longarg = templong;
		return 1;
	}
	printf("(getlongarg) \"%s\" is not a legal numeric input\n",
			instring);
	errexit();
	return -1;
}



/*
 * sends a pattern (fixed or user-defined depending
 * on who/how it gets called)
 *
 * global argarray contains contains byte values -
 * max of 16 data bytes + 1 byte pattern count
 *
 * number of things in array in global argcount
 */

static int sendpattern(void)
{
	time_t starttime;
	long int size, count;
	int bytesperpat;
	long int repeatcount;
	long int maxrepsperbuff;
	long int repsperbuff;
	long int buffbytesused;
	int i, j;

	bytesperpat = argcount - 1;
	repeatcount = argarray[argcount - 1];
	/*
	 * stuff buffer w/pattern rather than sending
	 * each pattern separately - to avoid giant
	 * packet overhead
	 */
	maxrepsperbuff = LOCBUFFSIZE/bytesperpat;
	repsperbuff = (repeatcount > maxrepsperbuff) ? maxrepsperbuff : repeatcount;
	buffbytesused = repsperbuff * bytesperpat;
	if(verbose) {
		printf("(sendpattern) bytes per patern = %d, repeat count = %d\n",
				bytesperpat, repeatcount);
		printf("(sendpattern) max patterns per buffer = %d, actual = %d\n",
				maxrepsperbuff, repsperbuff);
	}
	for(i=0; i<repsperbuff; i++)
		for(j=0; j<bytesperpat; j++)
			locbuffptr[(i * bytesperpat) + j] = (char)(argarray[j] & 0xFF);
	size = (argcount - 1) * argarray[argcount -1];
	if(verbose)
		printf("(sendpattern) sending %d byte(s), one '*' per senddata call\n",
				size);
	if(checkconnect())
		return -1;
	starttime = time(NULL);
	while(size) {
		count = (size > buffbytesused) ? buffbytesused : size;
		if(senddata(locbuffptr, count))
			return -1;
		size -= count;
		if(verbose) {
			printf("*");
			fflush(stdout);
		}
	}
	if(verbose)
		printf("\n(sendpattern) %d bytes sent in %d seconds\n",
				size, time(NULL) - starttime);
	return 0;
}



/*
 * send 0x55 0xaa test pattern - no wrapper
 */

static int sendtestpattern(void)
{
	unsigned long count, size;
	char ctemp;
	int i;
	time_t starttime;

	if(getlongarg(loptarg, NULL) <= 0){
		printf("(sendpattern) numeric count argument required\n");
		errexit();
		return -1;
	}
	argcount = 3;
	argarray[2] = longarg;
	argarray[0] = 0x55;
	argarray[1] = 0xaa;
	if(sendpattern()) {
		printf("(sendtestpattern) sendpattern error\n");
		errexit();
		return -1;
	}
	return 0;
}



/*
 * send byte or bytes
 * if single arg, send that byte once
 * if two args, 2nd is a repeat count
 * if more, all but last are byte pattern
 * to be repeated - last is count (# of pattern,
 * not total byte count)
 */

static int sendbytes(void)
{
	int getlongresult;
	char *endpointer;

	/* parse the arg list first */
	endpointer = loptarg;
	argcount = 0;
	while(1) {
		getlongresult = getlongarg(endpointer, &endpointer);
		if(getlongresult == -1) {
			printf("(sendbytes) getlongarg error\n");
			errexit();
			return -1;
		}
		if(getlongresult == 0) {
			if(verbose)
				printf("(sendbytes) end of arg list\n");
			break;
		}
		argarray[argcount] = longarg;
		if(verbose)
			printf("(sendbytes) arg # (from 0) %d is %d\n", argcount, longarg);
		argcount++;
		if(argcount >= (MAXPATBYTES + 2)) {
			printf("(sendbytes) too many arguments\n");
			errexit();
			return -1;
		}
	}
	if(verbose)
		printf("(sendbytes) %d argument(s) read\n", argcount);

	if(argcount == 0) {
		printf("(sendbytes) at least one argument required\n");
		errexit();
		return -1;
	}
	/*
	 * fudge the array if only one argument
	 * just output one byte
	 *
	 * then pass it all off to sendpattern
	 * which will digest the argarray contents
	 */
	if(argcount == 1) {
		argarray[1] = 1;
		argcount = 2;
	}
	if(sendpattern()) {
		printf("(sendbytes) sendpattern error\n");
		errexit();
		return -1;
	}
	return 0;
}



/*
 * send command or mode change via escape codes
 * a massive pile of string compares
 *
 * arg existence checking was done elsewhere
 *
 * just ripple through all the string comps
 * even when match found...
 */

static int sendcmdmode(void)
{
	int opcommand = -1;

	if(optopt == 'c') {
		/* speeds are inverted */
		if(!strcmp(loptarg, "SPEED_0"))
			opcommand = SPD3;
		if(!strcmp(loptarg, "SPEED_1"))
			opcommand = SPD2;
		if(!strcmp(loptarg, "SPEED_2"))
			opcommand = SPD1;
		if(!strcmp(loptarg, "SPEED_3"))
			opcommand = SPD0;
		if(!strcmp(loptarg, "VPRINT"))
			opcommand = PRINT;
		if(!strcmp(loptarg, "VPLOT"))
			opcommand = PLOT;
		if(!strcmp(loptarg, "VSPP_PRINT"))
			opcommand = SPRINT;
		if(!strcmp(loptarg, "VSPP_PLOT"))
			opcommand = SPLOT;
		if(!strcmp(loptarg, "VRST"))
			opcommand = RESET;
		if(!strcmp(loptarg, "VCLR"))
			opcommand = CLEAR;
		if(!strcmp(loptarg, "VFED"))
			opcommand = FF;
		if(!strcmp(loptarg, "VEOT"))
			opcommand = EOT;
		if(!strcmp(loptarg, "VLTR"))
			opcommand = LTR;
	}
	if(optopt == 'm') {
		if(!strcmp(loptarg, "PRINT"))
			opcommand = PRINT;
		if(!strcmp(loptarg, "PLOT"))
			opcommand = PLOT;
		if(!strcmp(loptarg, "SPP_PRINT"))
			opcommand = SPRINT;
		if(!strcmp(loptarg, "SPP_PLOT"))
			opcommand = SPLOT;
	}
	if(opcommand == -1) {
		printf("(sendcmdmode) \"%s\" is not valid sub-command/mode for \"%c\" command\n",
				loptarg, optopt);
		errexit();
		return -1;
	}
	if(checkconnect())
		return -1;
	*locbuffptr       = BA_ESC;
	*(locbuffptr + 1) = (char)opcommand;
	if(senddata(locbuffptr, 2)) {
		printf("(sendcmdmode) senddata error sending \"%s\" command\n",
				loptarg);
		errexit();
		return -1;
	}
	if(verbose)
		printf("(sendcommand) \"%s\" command sent\n", loptarg);
	return 0;
}



int main(int argc, char **argv)
{
	unsigned int local_arg;

	char linestr[132];
	char *lineptr;
	int c;
	int i;
	unsigned long count;
	char ctemp;

#ifdef _WINBUILD_
	/* WSA data cookie */
	WSADATA wsaData;
#endif

	/*
	 * hi mom, it's me!
	 */

	printf("\n%s\n%s\n%s\n%s\n\n", WINDEX_VERSION, WINDEX_BANNER, TAHOMA_BANNER,
			TAHOMA_CONTACT);

	/*
	 * require at least one command/option
	 */

	if(argc == 1) {
		usage();
		exit(EXIT_FAILURE);
	}

	locbuffptr = malloc(LOCBUFFSIZE);
	if(malloc == NULL) {
		printf("can't allocate local buffer\n");
		exit(EXIT_FAILURE);
	}

#ifdef _WINBUILD_
	winsockstart();
	printf("Windows socket services started\n");
#endif

	/*
	 * get command line or interactive commands
	 * leading ':' means return ':' if missing arg to
	 * option requiring arg
	 */

	while (1) {
		c = getcommand(argc, argv, ":ihqa:d:m:f:r:b:t:c:s:x:vnp");
		if(c != EOF) {
			switch (c) {
			case 'q':
				if(verbose)
					printf("command \"%c\" is \"quit\"\n", c);
				cleanup();
				printf("goodbye...\n");
				exit(EXIT_SUCCESS);
				break;
			case 'i':
				if(verbose)
					printf("command \"%c\" is \"interactive\"\n", c);
				if(verbose)
					printf("interactive mode set\n");
				interactive = 1;
				break;
			case 'h':
				if(verbose)
					printf("command \"%c\" is \"help\"\n", c);
				usage();
				break;
			case 'd':
				if(verbose)
					printf("command \"%c\" is \"set device port %s\"\n",
							c, loptarg);
				errno = 0;
				if(getlongarg(loptarg, NULL) <= 0) {
					printf("Invalid port number: %s\n", loptarg);
					errexit();
				}
				if(longarg > 0xffff) {
					printf("Port number out of range: %s\n", loptarg);
					errexit();
				}
				dev_port = (unsigned short)longarg;
				break;
			case 'a':
				if(verbose)
					printf("command \"%c\" is \"open device address %s\"\n",
							c, loptarg);
				/*
				 * open a socket and connect to addressed device
				 * just break if it returns - w/success or error
				 * fcns will abort w/exit if not interactive mode
				 */
				socketconnect();
				break;
			case 'f':
				if(verbose)
					printf("command \"%c\" is \"send wrapped file %s\"\n",
							c, loptarg);
				/*
				 * send file using pass-for-count
				 * escape wrapper
				 */
				sendwrappedfile();
				break;
			case 'r':
				if(verbose)
					printf("command \"%c\" is \"send raw file %s\"\n",
							c, loptarg);
				/*
				 * send raw data file - if any 0xBA
				 * chars in file, result is unpredictable
				 * (unless escapes are intentional...)
				 */
				sendrawfile();
				break;
			case 't':
				if(verbose)
					printf("command \"%c\" is \"send %s test pattern bytes\"\n",
							c, loptarg);
				/*
				 * send <count arg> 0x55, 0xAA bytes
				 */
				sendtestpattern();
				break;
			case 'c':
				if(verbose)
					printf("command \"%c\" is \"send %s sub-command\"\n",
							c, loptarg);
				/*
				 * send sub-command
				 */
				sendcmdmode();;
				break;
			case 'm':
				if(verbose)
					printf("command \"%c\" is \"set %s mode\"\n",
							c, loptarg);
				/*
				 * set mode
				 */
				sendcmdmode();;
				break;
			case 'b':
				if(verbose)
					printf("command %c is *send \"%s\" byte(s)*\n",
							c, loptarg);
				sendbytes();
				break;
			case 'v':
				if(verbose)
					printf("command \"%c\" is \"toggle verbose mode\"\n", c);
				if(verbose) {
					printf("verbose = 0\n");
					verbose = 0;
				} else {
					printf("verbose = 1\n");
					verbose = 1;
				}
				break;
			case 's':
				if(verbose)
					printf("command \"%c\" is \"send \'%s\' string\"\n",
							c, loptarg);
				if(checkconnect())
					break;
				if(senddata(loptarg, strlen(loptarg)))
					printf("send string error\n");
				break;
			case 'n':
				if(verbose)
					printf("command \"%c\" is \"send CR/LF (newline)\"\n", c);
				if(checkconnect())
					break;
				*locbuffptr       = '\r';
				*(locbuffptr + 1) = '\n';
				if(senddata(locbuffptr, 2))
					printf("send CR/LF error\n");
				break;
			case 'p':
				if(verbose)
					printf("command \"%c\" is \"send CR/FF (form feed)\"\n", c);
				if(checkconnect())
					break;
				*locbuffptr       = '\r';
				*(locbuffptr + 1) = '\f';
				if(senddata(locbuffptr, 2))
					printf("send CR/FF error\n");
				break;
			case ':':
				printf("Missing argument for option \"%c\"\n", optopt);
				errexit();
				break;	
			case '?':
				printf("Invalid option: \"%c\"\n", optopt);
				errexit();
				break;

			default:
				printf("Unrecognized option: \"%c\"\n", optopt);
errexit();
				break;
			}
		} else {
			if(verbose)
				printf("End of argument list\n");
			cleanup();
			break;
		}
	}
	return 0;
}

